from setuptools import setup, find_packages

setup(
    name="textarena",
    version="0.1.0",
    url="https://github.com/LeonGuertler/TextArena",
    author="Leon Guertler",
    author_email="Guertlerlo@cfar.a-star.edu.sg",
    description="[WIP] A Collection of Competitive Text-Based Games for Language Model Evaluation and Reinforcement Learning",
    packages=find_packages(),
    install_requires=[],  # fill in later
)
